@@ -18,7 +18,7 @@
 //!
 //! # #[tokio::main] async fn main() -> Result<(), Box<dyn std::error::Error>> {
 //! let token = env::var("DISCORD_TOKEN")?;
-//! let mut shard = Shard::new(token, Intents::GUILD_MESSAGES);
+//! let shard = Shard::new(token, Intents::GUILD_MESSAGES);
 //! shard.start().await?;
 //!
 //! // Create a cache, caching up to 10 messages per channel:
@@ -47,7 +47,13 @@
 //! [license link]: https://github.com/twilight-rs/twilight/blob/main/LICENSE.md
 //! [rust badge]: https://img.shields.io/badge/rust-1.49+-93450a.svg?style=for-the-badge&logo=rust
 
-#![deny(rust_2018_idioms, broken_intra_doc_links, unused, warnings)]
+#![deny(
+    clippy::missing_const_for_fn,
+    broken_intra_doc_links,
+    rust_2018_idioms,
+    unused,
+    warnings
+)]
 
 pub mod model;
 
@@ -72,10 +78,10 @@ use std::{
     sync::{Arc, Mutex},
 };
 use twilight_model::{
-    channel::{Group, GuildChannel, PrivateChannel},
+    channel::{Group, GuildChannel, PrivateChannel, StageInstance},
     gateway::presence::{Presence, UserOrId},
-    guild::{Emoji, Guild, Member, PartialMember, Role},
-    id::{ChannelId, EmojiId, GuildId, MessageId, RoleId, UserId},
+    guild::{Emoji, Guild, GuildIntegration, Member, PartialMember, Role},
+    id::{ChannelId, EmojiId, GuildId, IntegrationId, MessageId, RoleId, StageId, UserId},
     user::{CurrentUser, User},
     voice::VoiceState,
 };
@@ -145,13 +151,17 @@ struct InMemoryCacheRef {
     guilds: DashMap<GuildId, Arc<CachedGuild>>,
     guild_channels: DashMap<GuildId, HashSet<ChannelId>>,
     guild_emojis: DashMap<GuildId, HashSet<EmojiId>>,
+    guild_integrations: DashMap<GuildId, HashSet<IntegrationId>>,
     guild_members: DashMap<GuildId, HashSet<UserId>>,
     guild_presences: DashMap<GuildId, HashSet<UserId>>,
     guild_roles: DashMap<GuildId, HashSet<RoleId>>,
+    guild_stage_instances: DashMap<GuildId, HashSet<StageId>>,
+    integrations: DashMap<(GuildId, IntegrationId), GuildItem<GuildIntegration>>,
     members: DashMap<(GuildId, UserId), Arc<CachedMember>>,
     messages: DashMap<ChannelId, VecDeque<Arc<CachedMessage>>>,
     presences: DashMap<(GuildId, UserId), Arc<CachedPresence>>,
     roles: DashMap<RoleId, GuildItem<Role>>,
+    stage_instances: DashMap<StageId, GuildItem<StageInstance>>,
     unavailable_guilds: DashSet<GuildId>,
     users: DashMap<UserId, (Arc<User>, BTreeSet<GuildId>)>,
     /// Mapping of channels and the users currently connected.
@@ -229,7 +239,7 @@ impl InMemoryCache {
     }
 
     /// Create a new builder to configure and construct an in-memory cache.
-    pub fn builder() -> InMemoryCacheBuilder {
+    pub const fn builder() -> InMemoryCacheBuilder {
         InMemoryCacheBuilder::new()
     }
 
@@ -380,6 +390,19 @@ impl InMemoryCache {
         self.0.guild_roles.get(&guild_id).map(|r| r.value().clone())
     }
 
+    /// Gets the set of stage instances in a guild.
+    ///
+    /// This is a O(m) operation, where m is the amount of stage instances in
+    /// the guild. This requires the [`GUILDS`] intent.
+    ///
+    /// [`GUILDS`]: twilight_model::gateway::Intents::GUILDS
+    pub fn guild_stage_instances(&self, guild_id: GuildId) -> Option<HashSet<StageId>> {
+        self.0
+            .guild_stage_instances
+            .get(&guild_id)
+            .map(|r| r.value().clone())
+    }
+
     /// Gets a member by guild ID and user ID.
     ///
     /// This is an O(1) operation. This requires the [`GUILD_MEMBERS`] intent.
@@ -448,6 +471,18 @@ impl InMemoryCache {
             .map(|role| Arc::clone(&role.data))
     }
 
+    /// Gets a stage instance by ID.
+    ///
+    /// This is an O(1) operation. This requires the [`GUILDS`] intent.
+    ///
+    /// [`GUILDS`]: twilight_model::gateway::Intents::GUILDS
+    pub fn stage_instance(&self, stage_id: StageId) -> Option<Arc<StageInstance>> {
+        self.0
+            .stage_instances
+            .get(&stage_id)
+            .map(|role| Arc::clone(&role.data))
+    }
+
     /// Gets a user by ID.
     ///
     /// This is an O(1) operation. This requires the [`GUILD_MEMBERS`] intent.
@@ -504,9 +539,12 @@ impl InMemoryCache {
         self.0.guilds.clear();
         self.0.guild_channels.clear();
         self.0.guild_emojis.clear();
+        self.0.guild_integrations.clear();
         self.0.guild_members.clear();
         self.0.guild_presences.clear();
         self.0.guild_roles.clear();
+        self.0.guild_stage_instances.clear();
+        self.0.integrations.clear();
         self.0.members.clear();
         self.0.messages.clear();
         self.0.presences.clear();
@@ -671,6 +709,14 @@ impl InMemoryCache {
             self.cache_voice_states(guild.voice_states);
         }
 
+        if self.wants(ResourceType::STAGE_INSTANCE) {
+            self.0
+                .guild_stage_instances
+                .insert(guild.id, HashSet::new());
+            self.cache_stage_instances(guild.id, guild.stage_instances);
+        }
+
+        #[allow(deprecated)]
         let guild = CachedGuild {
             id: guild.id,
             afk_channel_id: guild.afk_channel_id,
@@ -690,7 +736,8 @@ impl InMemoryCache {
             member_count: guild.member_count,
             mfa_level: guild.mfa_level,
             name: guild.name,
-            nsfw: guild.nsfw,
+            nsfw: false,
+            nsfw_level: guild.nsfw_level,
             owner: guild.owner,
             owner_id: guild.owner_id,
             permissions: guild.permissions,
@@ -713,6 +760,21 @@ impl InMemoryCache {
         self.0.guilds.insert(guild.id, Arc::new(guild));
     }
 
+    fn cache_integration(&self, guild_id: GuildId, integration: GuildIntegration) {
+        self.0
+            .guild_integrations
+            .entry(guild_id)
+            .or_default()
+            .insert(integration.id);
+
+        upsert_guild_item(
+            &self.0.integrations,
+            guild_id,
+            (guild_id, integration.id),
+            integration,
+        );
+    }
+
     fn cache_member(&self, guild_id: GuildId, member: Member) -> Arc<CachedMember> {
         let member_id = member.user.id;
         let id = (guild_id, member_id);
@@ -834,6 +896,35 @@ impl InMemoryCache {
         upsert_guild_item(&self.0.roles, guild_id, role.id, role)
     }
 
+    fn cache_stage_instances(
+        &self,
+        guild_id: GuildId,
+        stage_instances: impl IntoIterator<Item = StageInstance>,
+    ) {
+        for stage_instance in stage_instances {
+            self.cache_stage_instance(guild_id, stage_instance);
+        }
+    }
+
+    fn cache_stage_instance(
+        &self,
+        guild_id: GuildId,
+        stage_instance: StageInstance,
+    ) -> Arc<StageInstance> {
+        self.0
+            .guild_stage_instances
+            .entry(guild_id)
+            .or_default()
+            .insert(stage_instance.id);
+
+        upsert_guild_item(
+            &self.0.stage_instances,
+            guild_id,
+            stage_instance.id,
+            stage_instance,
+        )
+    }
+
     fn cache_user(&self, user: Cow<'_, User>, guild_id: Option<GuildId>) -> Arc<User> {
         match self.0.users.get_mut(&user.id) {
             Some(mut u) if *u.0 == *user => {
@@ -962,6 +1053,19 @@ impl InMemoryCache {
         Some(data)
     }
 
+    fn delete_integration(&self, guild_id: GuildId, integration_id: IntegrationId) {
+        if self
+            .0
+            .integrations
+            .remove(&(guild_id, integration_id))
+            .is_some()
+        {
+            if let Some(mut integrations) = self.0.guild_integrations.get_mut(&guild_id) {
+                integrations.remove(&integration_id);
+            }
+        }
+    }
+
     fn delete_role(&self, role_id: RoleId) -> Option<Arc<Role>> {
         let role = self.0.roles.remove(&role_id).map(|(_, v)| v)?;
 
@@ -972,6 +1076,20 @@ impl InMemoryCache {
         Some(role.data)
     }
 
+    fn delete_stage_instance(&self, stage_id: StageId) -> Option<Arc<StageInstance>> {
+        let stage_instance = self.0.stage_instances.remove(&stage_id).map(|(_, v)| v)?;
+
+        if let Some(mut stage_instances) = self
+            .0
+            .guild_stage_instances
+            .get_mut(&stage_instance.guild_id)
+        {
+            stage_instances.remove(&stage_id);
+        }
+
+        Some(stage_instance.data)
+    }
+
     /// Determine whether the configured cache wants a specific resource to be
     /// processed.
     fn wants(&self, resource_type: ResourceType) -> bool {
@@ -979,7 +1097,7 @@ impl InMemoryCache {
     }
 }
 
-fn presence_user_id(presence: &Presence) -> UserId {
+const fn presence_user_id(presence: &Presence) -> UserId {
     match presence.user {
         UserOrId::User(ref u) => u.id,
         UserOrId::UserId { id } => id,
@@ -991,13 +1109,18 @@ mod tests {
     use crate::InMemoryCache;
     use std::borrow::Cow;
     use twilight_model::{
-        channel::{ChannelType, GuildChannel, TextChannel},
-        gateway::payload::{GuildEmojisUpdate, MemberRemove, RoleDelete},
+        channel::{
+            stage_instance::PrivacyLevel, ChannelType, GuildChannel, StageInstance, TextChannel,
+        },
+        gateway::payload::{
+            GuildEmojisUpdate, MemberRemove, RoleDelete, StageInstanceCreate, StageInstanceDelete,
+            StageInstanceUpdate,
+        },
         guild::{
             DefaultMessageNotificationLevel, Emoji, ExplicitContentFilter, Guild, Member, MfaLevel,
-            Permissions, PremiumTier, Role, SystemChannelFlags, VerificationLevel,
+            NSFWLevel, Permissions, PremiumTier, Role, SystemChannelFlags, VerificationLevel,
         },
-        id::{ChannelId, EmojiId, GuildId, RoleId, UserId},
+        id::{ChannelId, EmojiId, GuildId, RoleId, StageId, UserId},
         user::{CurrentUser, User},
         voice::VoiceState,
     };
@@ -1129,6 +1252,7 @@ mod tests {
             topic: None,
         })]);
 
+        #[allow(deprecated)]
         let guild = Guild {
             id: GuildId(123),
             afk_channel_id: None,
@@ -1152,6 +1276,7 @@ mod tests {
             mfa_level: MfaLevel::Elevated,
             name: "this is a guild".to_owned(),
             nsfw: false,
+            nsfw_level: NSFWLevel::AgeRestricted,
             owner: Some(false),
             owner_id: UserId(456),
             permissions: Some(Permissions::SEND_MESSAGES),
@@ -1162,6 +1287,7 @@ mod tests {
             region: "us-east".to_owned(),
             roles: Vec::new(),
             splash: None,
+            stage_instances: Vec::new(),
             system_channel_id: None,
             system_channel_flags: SystemChannelFlags::SUPPRESS_JOIN_NOTIFICATIONS,
             rules_channel_id: None,
@@ -1246,6 +1372,61 @@ mod tests {
         assert!(!cache.0.users.contains_key(&user_id));
     }
 
+    #[test]
+    fn test_stage_channels() {
+        let cache = InMemoryCache::new();
+
+        let stage_instance = StageInstance {
+            channel_id: ChannelId(1),
+            discoverable_disabled: true,
+            guild_id: GuildId(2),
+            id: StageId(3),
+            privacy_level: PrivacyLevel::GuildOnly,
+            topic: "topic".into(),
+        };
+
+        cache.update(&StageInstanceCreate(stage_instance.clone()));
+
+        {
+            let cached_instances = cache
+                .guild_stage_instances(stage_instance.guild_id)
+                .unwrap();
+            assert_eq!(1, cached_instances.len());
+        }
+
+        {
+            let cached_instance = cache.stage_instance(stage_instance.id).unwrap();
+            assert_eq!(stage_instance.topic, cached_instance.topic);
+        }
+
+        let new_stage_instance = StageInstance {
+            topic: "a new topic".into(),
+            ..stage_instance
+        };
+
+        cache.update(&StageInstanceUpdate(new_stage_instance.clone()));
+
+        {
+            let cached_instance = cache.stage_instance(stage_instance.id).unwrap();
+            assert_ne!(stage_instance.topic, cached_instance.topic);
+            assert_eq!(new_stage_instance.topic, "a new topic");
+        }
+
+        cache.update(&StageInstanceDelete(new_stage_instance));
+
+        {
+            let cached_instances = cache
+                .guild_stage_instances(stage_instance.guild_id)
+                .unwrap();
+            assert_eq!(0, cached_instances.len());
+        }
+
+        {
+            let cached_instance = cache.stage_instance(stage_instance.id);
+            assert_eq!(cached_instance, None);
+        }
+    }
+
     #[test]
     fn test_voice_state_inserts_and_removes() {
         let cache = InMemoryCache::new();
